@@ -13,6 +13,7 @@ from cis.utils import isnan, guess_coord_axis
 from cis.exceptions import ClassNotFoundError, CoordinateNotFoundError
 from cis.aggregation.aggregation_kernels import MultiKernel
 from cis.data_io.gridded_data import GriddedDataList
+from functools import reduce
 
 
 class Aggregator(object):
@@ -62,15 +63,28 @@ class Aggregator(object):
 
         return new_dims
 
-    def _gridded_full_collapse(self, coords, kernel):
+    @staticmethod
+    def _update_aux_factories(data, *args, **kwargs):
+        from cis.utils import listify
+        d_list = listify(data)
+        for d in d_list:
+            for factory in d.aux_factories:
+                factory.update(*args, **kwargs)
 
+    def _gridded_full_collapse(self, coords, kernel):
+        from cis.utils import demote_warnings
+        from copy import deepcopy
         ag_args = {}
         if isinstance(kernel, iris.analysis.WeightedAggregator):
             # If this is a list we can calculate weights using the first item (all variables should be on
             # same grid)
             data_for_weights = self.data[0] if isinstance(self.data, list) else self.data
             # Weights to correctly calculate areas.
-            ag_args['weights'] = iris.analysis.cartography.area_weights(data_for_weights)
+            logging.info("Calculating area weights for gridded collapse")
+            # Iris throws warnings about which radius it's using to calculate the weights, but these are really just
+            #  INFO level things.
+            with demote_warnings():
+                ag_args['weights'] = iris.analysis.cartography.area_weights(data_for_weights)
         elif not isinstance(kernel, iris.analysis.Aggregator):
             raise ClassNotFoundError('Error - unexpected aggregator type.')
 
@@ -89,10 +103,15 @@ class Aggregator(object):
                 # ... add it to our list of partial coordinates to collapse.
                 coords_for_partial_collapse.append((coord, coord_dims))
 
-        for coord, _ in coords_for_partial_collapse:
-            self.data.remove_coord(coord)
+        # Before we remove the coordinates which need to be partially collapsed we take a copy of the data. We need
+        #  this so that the aggregation doesn't have any side effects on the input data. This is particularly important
+        #  when using a MultiKernel for which this routine gets called multiple times.
+        data_for_collapse = deepcopy(self.data)
 
-        new_data = self.data.collapsed(coords, kernel, **ag_args)
+        for coord, _ in coords_for_partial_collapse:
+            data_for_collapse.remove_coord(coord)
+
+        new_data = data_for_collapse.collapsed(coords, kernel, **ag_args)
 
         for coord, old_dims in coords_for_partial_collapse:
             collapsed_coord = Aggregator._partially_collapse_multidimensional_coord(coord, dims_to_collapse)
@@ -101,7 +120,7 @@ class Aggregator(object):
             new_data.add_aux_coord(collapsed_coord, new_dims)
             # If the coordinate we had to collapse manually was a dependency in an aux factory (which is quite likely)
             #  then we need to put it back in and fix the factory, this will update any missing dependencies.
-            new_data.update_aux_factories(None, collapsed_coord)
+            self._update_aux_factories(new_data, None, collapsed_coord)
 
         return new_data
 
@@ -165,7 +184,7 @@ class Aggregator(object):
 
         if len(self._grid) != 0:
             raise CoordinateNotFoundError("No coordinate found that matches '{}'. Please check the coordinate "
-                                          "name.".format("' or '".join(self._grid.keys())))
+                                          "name.".format("' or '".join(list(self._grid.keys()))))
 
         dummy_data = np.reshape(np.arange(int(np.prod(new_cube_shape))) + 1.0, tuple(new_cube_shape))
         aggregation_cube = iris.cube.Cube(dummy_data, dim_coords_and_dims=new_cube_coords)
@@ -203,7 +222,7 @@ class Aggregator(object):
         try:
             coord_unit = as_unit(coord.units)
         except ValueError as e:
-            if e.message.startswith('[UT_UNKNOWN]') and coord.standard_name in coordinate_unit_mappings:
+            if e.args[0].startswith('[UT_UNKNOWN]') and coord.standard_name in coordinate_unit_mappings:
                 # For some common coordinates we can fix this...
                 coord_unit = as_unit(coordinate_unit_mappings[coord.standard_name])
                 logging.warning("Converting units for {coord} from {old} to the CF-compliant units: {new}"
