@@ -2,19 +2,36 @@ from time import gmtime, strftime
 import logging
 
 import iris
-from iris.cube import CubeList
 import numpy as np
 
 from cis.data_io.common_data import CommonData, CommonDataList
 from cis.data_io.hyperpoint import HyperPoint
 from cis.data_io.hyperpoint_view import GriddedHyperPointView
 
-from iris.std_names import STD_NAMES
-from cis.utils import remove_file_prefix
-
 
 def load_cube(*args, **kwargs):
-    iris_cube = iris.load_cube(*args, **kwargs)
+    """
+    Load a single GriddedData object through the iris load interface, but also attempt concatenation if merging fails
+
+    :return GriddedData: A single GriddedData object
+    :raises ValueError: If 0 or more than one cube is found
+    """
+    from iris.exceptions import MergeError, ConcatenateError
+    # Removes warnings and prepares for future Iris change
+    iris.FUTURE.netcdf_promote = True
+
+    cubes = iris.load(*args, **kwargs)
+    try:
+        iris_cube = cubes.merge_cube()
+    except MergeError as e:
+        logging.info("Unable to merge cubes on load: \n {}\nAttempting to concatenate instead.".format(e))
+        try:
+            iris_cube = cubes.concatenate_cube()
+        except ConcatenateError as e:
+            logging.error("Unable to concatenate cubes on load: \n {}".format(e))
+            raise ValueError("Unable to create a single cube from arguments given: {}".format(args))
+    except ValueError as e:
+        raise ValueError("No cubes found")
     return make_from_cube(iris_cube)
 
 
@@ -240,16 +257,16 @@ class GriddedData(iris.cube.Cube, CommonData):
         :param attributes: Dictionary of attribute names (keys) and values.
         :return:
         """
-        for key, value in attributes.items():
+        for key, value in list(attributes.items()):
             try:
                 self.attributes[key] = value
             except ValueError:
                 try:
                     setattr(self, key, value)
                 except ValueError as e:
-                    logging.warning("Could not set NetCDF attribute '%s' because %s" % (key, e.message))
+                    logging.warning("Could not set NetCDF attribute '%s' because %s" % (key, e.args[0]))
         # Record that this is a local (variable) attribute, not a global attribute
-        self._local_attributes.extend(attributes.keys())
+        self._local_attributes.extend(list(attributes.keys()))
 
     def remove_attribute(self, key):
         """
@@ -287,10 +304,6 @@ class GriddedData(iris.cube.Cube, CommonData):
         from iris.pandas import as_data_frame
         return as_data_frame(self, copy=copy)
 
-    def update_aux_factories(self, *args, **kwargs):
-        for factory in self.aux_factories:
-            factory.update(*args, **kwargs)
-
     def collapsed(self, *args, **kwargs):
         return make_from_cube(super(GriddedData, self).collapsed(*args, **kwargs))
 
@@ -496,7 +509,3 @@ class GriddedDataList(iris.cube.CubeList, CommonDataList):
         """
         # Use the dimensions of the first item since all items should be the same shape
         return self[0].ndim
-
-    def update_aux_factories(self, *args, **kwargs):
-        for d in self:
-            d.update_aux_factories(*args, **kwargs)
