@@ -8,29 +8,42 @@ import os.path
 import logging
 
 from cis.exceptions import InvalidCommandLineOptionError
-from plotting.plot import Plotter
-from utils import add_file_prefix
+from cis.plotting.plot import Plotter
 
 
 def initialise_top_parser():
     """
     The parser to which all arguments are initially passed
     """
-    parser = argparse.ArgumentParser("cis")
+    global_options = argparse.ArgumentParser(add_help=False)
+    verbosity_group = global_options.add_mutually_exclusive_group()
+    verbosity_group.add_argument("-v", "--verbose", action='count',
+                                 help="Increase the level of logging information output to screen to include "
+                                      "'Info' statements")
+    verbosity_group.add_argument("-q", "--quiet", action='store_true',
+                                 help="Suppress all output to the screen, only 'Error' messages will be displayed "
+                                      "(which are always fatal).")
+    global_options.add_argument("--force-overwrite", action='store_true',
+                                help="Do not prompt when an output file already exists - always overwrite. This can "
+                                     "also be set by setting the 'CIS_FORCE_OVERWRITE' environment variable to 'TRUE'")
+
+    parser = argparse.ArgumentParser("cis", parents=[global_options])
+
     subparsers = parser.add_subparsers(dest='command')
-    plot_parser = subparsers.add_parser("plot", help="Create plots")
+    plot_parser = subparsers.add_parser("plot", help="Create plots", parents=[global_options])
     add_plot_parser_arguments(plot_parser)
-    info_parser = subparsers.add_parser("info", help="Get information about a file")
+    info_parser = subparsers.add_parser("info", help="Get information about a file", parents=[global_options])
     add_info_parser_arguments(info_parser)
-    col_parser = subparsers.add_parser("col", help="Perform collocation")
+    col_parser = subparsers.add_parser("col", help="Perform collocation", parents=[global_options])
     add_col_parser_arguments(col_parser)
-    aggregate_parser = subparsers.add_parser("aggregate", help="Perform aggregation")
+    aggregate_parser = subparsers.add_parser("aggregate", help="Perform aggregation", parents=[global_options])
     add_aggregate_parser_arguments(aggregate_parser)
-    subset_parser = subparsers.add_parser("subset", help="Perform subsetting")
+    subset_parser = subparsers.add_parser("subset", help="Perform subsetting", parents=[global_options])
     add_subset_parser_arguments(subset_parser)
-    eval_parser = subparsers.add_parser("eval", help="Evaluate a numeric expression")
+    eval_parser = subparsers.add_parser("eval", help="Evaluate a numeric expression", parents=[global_options])
     add_eval_parser_arguments(eval_parser)
-    stats_parser = subparsers.add_parser("stats", help="Perform statistical comparison of two datasets")
+    stats_parser = subparsers.add_parser("stats", help="Perform statistical comparison of two datasets",
+                                         parents=[global_options])
     add_stats_parser_arguments(stats_parser)
     subparsers.add_parser("version", help="Display the CIS version number")
     return parser
@@ -52,7 +65,7 @@ def add_plot_parser_arguments(parser):
     parser.add_argument("-o", "--output", metavar="Output filename", nargs="?",
                         help="The filename of the output file for the plot image")
     parser.add_argument("--type", metavar="Chart type", nargs="?",
-                        help="The chart type, one of: " + str(Plotter.plot_types.keys()))
+                        help="The chart type, one of: " + str(list(Plotter.plot_types.keys())))
 
     parser.add_argument("--xlabel", metavar="X axis label", nargs="?", help="The label for the x axis")
     parser.add_argument("--ylabel", metavar="Y axis label", nargs="?", help="The label for the y axis")
@@ -122,11 +135,9 @@ def add_plot_parser_arguments(parser):
 
 
 def add_info_parser_arguments(parser):
-    parser.add_argument("filenames", metavar="Filenames", help="The filenames of the files to inspect", nargs='*')
-    parser.add_argument("-v", "--variables", metavar="Variable(s)", nargs="+", help="The variable(s) to inspect")
-    parser.add_argument("--product", metavar="The specific data product to use", nargs="?",
-                        help="CIS will try and automatically determine the best product to use, but this option can"
-                             "override CIS to specify a different data product to use for reading the data..")
+    parser.add_argument("datagroups", metavar="DataGroups", nargs=1,
+                        help="Variables and files to inspect, which needs to be entered in the format "
+                             "[variables:]filename[:product=].")
     parser.add_argument("--type", metavar="Type of HDF data", nargs="?",
                         help="Can be 'VD' or 'SD'. Use 'All' for both.")
     return parser
@@ -142,8 +153,7 @@ def add_col_parser_arguments(parser):
                              "specified. For example filename:variable=var1,collocator=box[h_sep=10km].")
     parser.add_argument("-o", "--output", metavar="Output filename", default="out", nargs="?",
                         help="The filename of the output file containing the collocated data. The name specified will"
-                             " be suffixed with \".nc\". For ungridded output, it will be prefixed with \"cis-\" and "
-                             "so that cis can recognise it when using the file for further operations.")
+                             " be suffixed with \".nc\".")
     return parser
 
 
@@ -285,7 +295,7 @@ def check_aggregate_kernel(arg, parser):
 
     aggregation_classes = plugin.find_plugin_classes(Kernel, 'cis.collocation.col_implementations')
     aggregation_names = [cls().__class__.__name__ for cls in aggregation_classes]
-    if arg in aggregation_kernels.keys() or arg in aggregation_names:
+    if arg in list(aggregation_kernels.keys()) or arg in aggregation_names:
         return arg
     else:
         parser.error(arg + " is not a valid aggregation kernel. Please use one of " + str(aggregation_names))
@@ -514,7 +524,7 @@ def parse_colon_and_comma_separated_arguments(inputs, parser, options, compulsor
 
         input_dict = {}
 
-        option = options._asdict().keys()
+        option = list(options._asdict().keys())
 
         # First deal with the comma separated compulsory arguments
         for i in range(0, compulsory_args):
@@ -699,9 +709,9 @@ def check_plot_type(plot_type, parser):
     """
 
     if plot_type is not None:
-        if plot_type not in Plotter.plot_types.keys():
+        if plot_type not in list(Plotter.plot_types.keys()):
             parser.error(
-                "'" + plot_type + "' is not a valid plot type, please use one of: " + str(Plotter.plot_types.keys()))
+                "'" + plot_type + "' is not a valid plot type, please use one of: " + str(list(Plotter.plot_types.keys())))
 
     return plot_type
 
@@ -804,22 +814,38 @@ def _split_output_if_includes_variable_name(arguments, parser):
             parser.error("Invalid output path: should be a filename with one optional variable prefix.")
 
 
-def _validate_output_file(arguments, parser):
+def _validate_output_file(arguments, parser, default_ext='.nc'):
     _split_output_if_includes_variable_name(arguments, parser)
-    _check_output_filepath_not_input(arguments, parser)
-    _append_file_extension_to_output_if_missing(arguments, parser, '.nc')
+    if not os.path.splitext(arguments.output)[1]:
+        arguments.output += default_ext
+    if _file_already_exists_and_no_overwrite(arguments):
+        parser.exit(status=0, message="No operation performed")
+    if _output_file_matches_an_input_file(arguments):
+        parser.error("The input file must not be the same as the output file")
 
 
-def _append_file_extension_to_output_if_missing(arguments, parser, extension):
-    if not extension:
-        parser.error("Invalid file extension: '%s'" % extension)
-    if not extension.startswith('.'):
-        extension = '.' + extension
-    if not arguments.output.endswith(extension):
-        arguments.output = arguments.output + extension
+def _file_already_exists_and_no_overwrite(arguments):
+    from six.moves import input
+    # If the file already exists, and we haven't set the overwrite flag or env var, then prompt
+    if os.path.isfile(arguments.output) and \
+            not (arguments.force_overwrite or os.environ.get("CIS_FORCE_OVERWRITE", "false").lower() == "true"):
+        overwrite = None
+        while overwrite not in ['y', 'n', '']:
+            overwrite = input("The file: {} already exists. Overwrite? (y/[n])")
+        if overwrite != 'y':
+            return True
+        # Otherwise False
+    return False
 
 
-def _check_output_filepath_not_input(arguments, parser):
+def _output_file_matches_an_input_file(arguments):
+    """
+    Checks that the output file is not also one of the input files. Uses os.path.samefile where available (not NT) or
+    falls back to comparing os.stat.
+    :param arguments: The command line arguments
+    :return: True if the output file is also an input file
+    """
+    match = False
     try:
         input_files = list(arguments.samplefiles)
     except AttributeError:
@@ -827,13 +853,14 @@ def _check_output_filepath_not_input(arguments, parser):
 
     for datagroup in arguments.datagroups:
         input_files.extend(datagroup['filenames'])
-    gridded_output_file = arguments.output + ".nc"
-    ungridded_output_file = add_file_prefix('cis-', gridded_output_file)
-    for output_file in [gridded_output_file, ungridded_output_file]:
-        for input_file in input_files:
-            if os.path.exists(output_file):
-                if os.path.samefile(output_file, input_file):
-                    parser.error("The input file must not be the same as the output file")
+    for input_file in input_files:
+        if os.path.exists(arguments.output):
+            # Windows doesn't have samefile support...
+            if (hasattr(os.path, 'samefile') and os.path.samefile(arguments.output, input_file)) or \
+                    (arguments.output == input_file and os.stat(arguments.output) == os.stat(input_file)):
+                match = True
+                break
+    return match
 
 
 def _create_attributes_dictionary(arguments, parser):
@@ -879,11 +906,21 @@ def validate_plot_args(arguments, parser):
     arguments.xbinwidth = parse_float(arguments.xbinwidth, "x bin width", parser)
     arguments.ybinwidth = parse_float(arguments.ybinwidth, "y bin width", parser)
 
+    if arguments.output is not None:
+        _validate_output_file(arguments, parser, '.png')
+
     return arguments
 
 
 def validate_info_args(arguments, parser):
-    arguments.filenames = expand_file_list(','.join(arguments.filenames), parser)
+    split_input = [re.sub(r'([\\]):', r':', word) for word in re.split(r'(?<!\\):', arguments.datagroups[0])]
+    if len(split_input) == 1:
+        # If there is only one part of the datagroup then it must be a file (or list of files). Return it as a dict to
+        #  match the behaviour of the datagroup parser.
+        arguments.datagroups = [{'filenames': expand_file_list(arguments.datagroups[0], parser),
+                                 'variables': None, 'product': None}]
+    else:
+        arguments.datagroups = get_basic_datagroups(arguments.datagroups, parser)
     return arguments
 
 
@@ -967,6 +1004,15 @@ def parse_args(arguments=None):
         # sys.argv[0] is the name of the script itself
         arguments = sys.argv[1:]
     main_args = parser.parse_args(arguments)
+    # Firstly deal with logging verbosity - in case we log anything in the validation
+    # The 'screen' handler is the first in the list
+    if main_args.quiet:
+        logging.getLogger().handlers[0].setLevel(logging.ERROR)
+    elif main_args.verbose == 1:
+        logging.getLogger().handlers[0].setLevel(logging.INFO)
+    elif main_args.verbose == 2:
+        logging.getLogger().handlers[0].setLevel(logging.DEBUG)
+
     main_args = validators[main_args.command](main_args, parser)
 
     return main_args
