@@ -2,7 +2,11 @@ import logging
 
 import numpy as np
 from matplotlib.ticker import MaxNLocator, AutoMinorLocator
-import matplotlib.pyplot as plt
+
+# Matplotlib at v1.5 keeps throwing warnings about building the font cache - these are INFO level at best.
+from cis.utils import demote_warnings
+with demote_warnings():
+    import matplotlib.pyplot as plt
 
 from cis.exceptions import CISError
 from cis.utils import find_longitude_wrap_start
@@ -448,24 +452,19 @@ class Generic_Plot(object):
         Calculates the min and max values of all the data given
         Stores the values in the matplotlib keyword args to be directly passed into the plot methods.
         """
-        from sys import maxint
-        from numpy import ma
+        vmin = self.plot_args["valrange"].get("vmin", None)
+        vmax = self.plot_args["valrange"].get("vmax", None)
 
-        if self.plot_args.get('logv', False):
-            mask_val = 0.0
-        else:
-            mask_val = -maxint - 1
-
-        vmin = self.plot_args["valrange"].get("vmin", maxint)
-        vmax = self.plot_args["valrange"].get("vmax", -maxint - 1)
-
-        if vmin == maxint:
-            vmin = min(ma.array(unpacked_data_item["data"], mask=unpacked_data_item["data"] <= mask_val).min() for
-                       unpacked_data_item in self.unpacked_data_items)
-
-        if vmax == -maxint - 1:
-            vmax = max(ma.array(unpacked_data_item["data"], mask=unpacked_data_item["data"] <= mask_val).max() for
-                       unpacked_data_item in self.unpacked_data_items)
+        if vmin is None:
+            if self.plot_args.get('logv', False):
+                vmin = min(d["data"][d["data"] > 0].min() for d in self.unpacked_data_items)
+            else:
+                vmin = min(d["data"].min() for d in self.unpacked_data_items)
+        if vmax is None:
+            if self.plot_args.get('logv', False):
+                vmax = max(d["data"][d["data"] > 0].max() for d in self.unpacked_data_items)
+            else:
+                vmax = max(d["data"].max() for d in self.unpacked_data_items)
 
         self.mplkwargs["vmin"] = float(vmin)
         self.mplkwargs["vmax"] = float(vmax)
@@ -561,7 +560,7 @@ class Generic_Plot(object):
             try:
                 self.matplotlib.gca().ticklabel_format(style='sci', scilimits=(-3, 3), axis='both')
             except AttributeError:
-                logging.warning("Couldn't apply scientific notation to axes")
+                logging.debug("Couldn't apply scientific notation to axes")
 
         draw_grid = self.plot_args.pop("grid", False)
         if draw_grid:
@@ -582,9 +581,9 @@ class Generic_Plot(object):
         if self.plot_args["title"] is None:
             self.plot_args["title"] = ""
 
-        for key in plot_options.keys():
+        for key in list(plot_options.keys()):
             # Call the method associated with the option
-            if key in self.plot_args.keys():
+            if key in list(self.plot_args.keys()):
                 plot_options[key](self.plot_args[key])
 
         if len(self.packed_data_items) > 1:
@@ -605,7 +604,7 @@ class Generic_Plot(object):
         :return: Can we access natural earth?
         """
         from cartopy.io.shapereader import natural_earth
-        from urllib2 import HTTPError
+        from six.moves.urllib.error import HTTPError
         try:
             natural_earth_available = natural_earth()
         except HTTPError:
@@ -682,9 +681,9 @@ class Generic_Plot(object):
         if self.plot_args["title"] is None:
             self.plot_args["title"] = self.packed_data_items[0].long_name
 
-        for key in plot_options.keys():
+        for key in list(plot_options.keys()):
             # Call the method associated with the option
-            if key in self.plot_args.keys():
+            if key in list(self.plot_args.keys()):
                 plot_options[key](self.plot_args[key])
 
         if not self.plot_args["nocolourbar"]:
@@ -866,7 +865,9 @@ class Generic_Plot(object):
             if y_variable.startswith(lat_or_lon):
                 lat_locator = MaxNLocator(nbins=max_y_bins, steps=lat_steps)
                 if self.is_map():
-                    self.cartopy_axis.set_yticks(lat_locator.tick_values(ymin, ymax), crs=self.transform)
+                    # Get best guess tick values - as long as they are in a sensible range...
+                    tick_values = [v for v in lat_locator.tick_values(ymin, ymax) if abs(v) <= 90.0]
+                    self.cartopy_axis.set_yticks(tick_values, crs=self.transform)
                     self.cartopy_axis.yaxis.set_major_formatter(LatitudeFormatter())
                 else:
                     self.matplotlib.axes().yaxis.set_major_locator(lat_locator)
