@@ -4,6 +4,8 @@ import logging
 import warnings
 import numpy as np
 from cis.exceptions import InvalidCommandLineOptionError
+import contextlib
+
 
 # number of bytes in a MB
 BYTES_IN_A_MB = 1048576.0
@@ -89,11 +91,10 @@ def calculate_histogram_bin_edges(data, axis, user_range, step, log_scale=False)
     return bin_edges
 
 
-def expand_1d_to_2d_array(array_1d, length, axis=0):
+def expand_1d_to_2d_array(array, length, axis=0):
     """
-    General utility routine to 'extend a 1D array into a 2D array
-    by duplicating the data along a given 'axis' (default is 0)
-    of size 'length'.
+    General utility routine to 'extend arbitrary dimensional array into a higher dimension
+    by duplicating the data along a given 'axis' (default is 0) of size 'length'.
 
     Examples::
 
@@ -111,17 +112,20 @@ def expand_1d_to_2d_array(array_1d, length, axis=0):
          [3 3 3 3]
          [4 4 4 4]]
 
-    :param array_1d:
+    :param array:
     :param length:
     :param axis:
     :return:
     """
-    import numpy as np
+    from numpy.lib.stride_tricks import broadcast_to
 
     if axis == 0:
-        array_2d = np.repeat(array_1d[np.newaxis, :], length, 0)
+        array_2d = broadcast_to(array, (length, array.size))
+        # array_2d = np.lib.stride_tricks.as_strided(array_1d, (length, array_1d.size), (0, array_1d.itemsize))
     else:
-        array_2d = np.repeat(array_1d[:, np.newaxis], length, 1)
+        reshaped = array.reshape(array.size, 1)
+        array_2d = broadcast_to(reshaped, (array.size, length))
+        # array_2d = np.lib.stride_tricks.as_strided(array_1d, (array_1d.size, length), (array_1d.itemsize, 0))
 
     return array_2d
 
@@ -136,7 +140,8 @@ def create_masked_array_for_missing_values(data, missing_values):
 
     mdata = data
     for missing_value in missing_values:
-        mdata = ma.masked_where(missing_value == mdata, mdata)
+        if missing_value is not None:
+            mdata = ma.masked_where(missing_value == mdata, mdata)
 
     return mdata
 
@@ -281,14 +286,14 @@ def unpack_data_object(data_object, x_variable, y_variable, x_wrap_start):
                         data, x = add_cyclic_point(data, x)
                     except ValueError as e:
                         logging.warn('Unable to add cyclic data point for {}. Error was: '.format(x_variable)
-                                     + e.message)
+                                     + e.args[0])
                     x, y = np.meshgrid(x, y)
                 elif len(y) == data.shape[-1]:
                     try:
                         data, y = add_cyclic_point(data, y)
                     except ValueError as e:
                         logging.warn('Unable to add cyclic data point for {}. Error was: '.format(y_variable)
-                                     + e.message)
+                                     + e.args[0])
                     y, x = np.meshgrid(y, x)
     elif x_axis_name == 'X' and x_wrap_start is not None:
         x = fix_longitude_range(x, x_wrap_start)
@@ -369,38 +374,6 @@ def copy_attributes(source, dest):
             dest.__dict__.update(source.__dict__)
 
 
-def add_file_prefix(prefix, filepath):
-    """
-    Add a prefix to a filename taking into account any path that might be present before that actual filename
-
-    :param prefix: A string to prefix the filename with
-    :param filepath: Filename, optionally including path
-    :return: A string with the full path to the prefixed file
-    """
-    import os.path
-    filename = os.path.basename(filepath)
-    path = os.path.dirname(filepath)
-    return os.path.join(path, prefix + filename)
-
-
-def remove_file_prefix(prefix, filepath):
-    """
-    Remove a prefix from a filename, taking into account any path that might be present before that actual filename
-
-    :param prefix: The prefix to remove
-    :param filepath: Filename, optional including path
-    :return: A string with the full path to the un-prefixed file
-    """
-    import os.path
-
-    filename = os.path.basename(filepath)
-    path = os.path.dirname(filepath)
-
-    filename = filename.replace(prefix, '', 1)
-
-    return os.path.join(path, filename)
-
-
 def parse_key_val_string(arguments, separator):
     """
     Takes a (comma) separated list of keyword value pairs (separated by =) and returns a dictionary with those keys and
@@ -717,9 +690,10 @@ def guess_coord_axis(coord):
     """
     import iris
     # TODO Can more be done for ungridded based on units, as with iris.util.guess_coord_axis?
+    guessed_axis = None
     if isinstance(coord, iris.coords.Coord):
         guessed_axis = iris.util.guess_coord_axis(coord)
-    else:
+    elif coord.standard_name is not None:
         guessed_axis = standard_names.get(coord.standard_name.lower())
     return guessed_axis
 
@@ -736,35 +710,34 @@ def add_to_list_if_not_none(item, list):
         list.append(item)
 
 
-def dimensions_equal(dimensions, other_dimensions):
+def dimensions_compatible(dimensions, other_dimensions):
     """
-    Check to see if two dimensions are the same (contain the same variables in the same order)
+    Check to see if two dimensions share the same common coordinates. Note that this will only compare the dimensions
+    up to the length of the shortest list.
 
     :param dimensions: dimension list
     :param other_dimensions: other dimension list
     """
 
-    if len(dimensions) is not len(other_dimensions):
-        return False
     for dim, other_dim in zip(dimensions, other_dimensions):
         if not dim == other_dim:
             return False
     return True
 
 
-def set_cube_standard_name_if_valid(cube, standard_name):
+def set_standard_name_if_valid(data, standard_name):
     """
-    Set a cube's standard name if it is a valid CF compliant name, otherwise set it to None
+    Set a data object's standard name if it is a valid CF compliant name, otherwise set it to None
 
-    :param cube: Cube to set standard name on
+    :param CommonData or Metadata data: Data to set standard name on
     :param standard_name: Standard name to set
     :return:
     """
     try:
-        cube.standard_name = standard_name
+        data.standard_name = standard_name
     except ValueError:
-        # If the standard name is not valid CF compliant standard name
-        cube.standard_name = None
+        logging.info("Not setting standard name '{}' as it is not CF compliant.".format(standard_name))
+        data.standard_name = None
 
 
 def deprecated(func):
@@ -819,3 +792,27 @@ def log_memory_profile(location):
         logging.debug("App Memory MB ({}): {}".format(location, mem))
     except ImportError:
         pass
+
+
+def move_item_to_end(iter, item):
+    """
+    Move an item in an iterable to the end of a list
+    :param iterable iter: iterable container (list or tuple) contianing item
+    :param item: item to move to end
+    :return list: rearranged list
+    """
+    if not isinstance(iter, list):
+        iter = list(iter)
+    dim = iter.pop(iter.index(item))
+    iter.append(dim)
+    return iter
+
+
+@contextlib.contextmanager
+def demote_warnings(level=logging.INFO):
+    import warnings
+    with warnings.catch_warnings(record=True) as ws:
+        warnings.simplefilter("always")
+        yield
+        for w in ws:
+            logging.log(level, w.message)
